#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <omp.h>
#include <mpi.h>
#include <signal.h>
#include <unistd.h>

// Scheme for the problem of Accumulations in matrix, MPI+OpenMP version
// Spanish Parallel Programming Contest 2013, problem B

void initialize(int *value,int *r1,int *c1,int *r2,int *c2,int M,int N,int lowervalue,int uppervalue)
{
  int i,temp;

// Integer numbers are generated randomly
  for(i=0;i<M;i++)
  {  
    value[i] =(int )(((1.*rand())/RAND_MAX)*(uppervalue-lowervalue)+lowervalue);
    while(value[i]==0)
    {
      value[i] =(int )(((1.*rand())/RAND_MAX)*(uppervalue-lowervalue)+lowervalue);
    }
// rows and columns are generated randomly
    r1[i] =(int) (((1.*rand())/RAND_MAX)*N);
    c1[i] =(int) (((1.*rand())/RAND_MAX)*N);
    r2[i] =(int) (((1.*rand())/RAND_MAX)*N);
    while(r2[i]==r1[i])
    {
      r2[i] =(int) (((1.*rand())/RAND_MAX)*N);
    }
    if(r2[i]<r1[i])
    {
      temp=r1[i];
      r1[i]=r2[i];
      r2[i]=temp;
    }
    c2[i] =(int) (((1.*rand())/RAND_MAX)*N);
    while(c2[i]==c1[i])
    {
      c2[i] =(int) (((1.*rand())/RAND_MAX)*N);
    }
    if(c2[i]<c1[i])
    {
      temp=c1[i];
      c1[i]=c2[i];
      c2[i]=temp;
    }
  }
}

void escribir(int *m, int t){
  int i, j;

  for (i = 0; i < t; i++)
  {
    for (j = 0; j < t; j++)
    {  
      printf("%d ",m[i*t+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void  escribirvalues(int *value,int *r1,int *c1,int *r2,int *c2,int M)
{
  int i;

  for (i = 0; i < M; i++)
  {
    printf("%d %d %d %d %d , ",value[i],r1[i],c1[i],r2[i],c2[i]);
  }
  printf("\n");
}

void escribirresult(int *a,int N,int salida)
{
  int i,j;

  for(i=0;i<N;i++)
  {
     for(j=0;j<N;j++)
     {
        if(((i*N+j)%salida)==0)
        {
          printf("%d ",a[i*N+j]);
        }
     }
     printf("\n");
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,int,int *,int *,int *,int *,int *,int *,int,int);

int main(int argc,char *argv[]) {
  int nodo,np;
  int i,j,N,M;
  int lowervalue,uppervalue;
  int cuantos,semilla,salida;
  long long ti,tf,tt=0;
  int *a,*value;
  int *r1,*c1,*r2,*c2;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&np);
  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
  if(nodo==0)
  {
    scanf("%d",&cuantos);
    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  else
  {
    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  for(i=0;i<cuantos;i++)
  {
    if(nodo==0)
    {
      scanf("%d",&N);                   // The first argument is the matrix size
      scanf("%d",&M);                   // The first argument is the number of elements to accumulate
      scanf("%d",&semilla);             // the third is the seed for random generation
      scanf("%d",&lowervalue);		//the lower value for the integers
      scanf("%d",&uppervalue);		// the maximum value of the ingeter; which will be generated between -lowervalue+1 and uppervalue-1
      scanf("%d",&salida);                 // to write selected values of the output matrix

// Space for the matrix, the values, rows and columns
      a = (int *) calloc(sizeof(int),N*N);
      value = (int *) malloc(sizeof(int)*M);
      r1=(int *) malloc(sizeof(int)*M);
      c1=(int *) malloc(sizeof(int)*M);
      r2=(int *) malloc(sizeof(int)*M);
      c2=(int *) malloc(sizeof(int)*M);

      srand(semilla);
      initialize(value,r1,c1,r2,c2,M,N,lowervalue,uppervalue);

#ifdef DEBUG
// While debugging the matrix and the valus and positions are written
      escribir(a,N);
      escribirvalues(value,r1,c1,r2,c2,M);
#endif
    }
    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,M,a,value,r1,c1,r2,c2,nodo,np);
    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
    if(nodo==0)
    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(a,N,salida);
      free(a);
      free(value);
      free(r1);
      free(c1);
      free(r2);
      free(c2);
    } 
  }
  
  if(nodo==0)
  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
  }
  MPI_Finalize();
  return 0;
}
