#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
//#include <mpi.h>
//#include <omp.h>
#include <signal.h>
#include <unistd.h>

// Scheme for the problem of Bidimaneional Taylor approximation, MPI+OpenMP version
// Spanich Parallel Programming Contest 2013, problem D

void initialize(double *pol,int N, double *meshx,double *meshy,int P,double lr,double ur,double lm,double um)
{
  int i,j;
  double space;

// Values in the polynomial are generated randomly
  for(i=0;i<N*N;i++)
  {  
    pol[i] =((1.*rand())/RAND_MAX)*(ur-lr)+lr;
  }
// Points in the mesh are generated equally spaced
  space=(um-lm)/(P-1);
  for(i=0;i<P;i++)
  {
    for(j=0;j<P;j++)
    {
      meshx[i*P+j]=lm+i*space;
    }
  }  
  for(i=0;i<P;i++)
  {
    for(j=0;j<P;j++)
    {
      meshy[i*P+j]=lm+j*space;
    }
  }  
}

void escribir(double *m, int t){
  int i, j;

  for (i = 0; i < t; i++)
  {
    for (j = 0; j < t; j++)
    {  
      printf("%.1lf ",m[i*t+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void escribirresult(double *a,int N,int salida)
{
  int i,j;

  for(i=0;i<N;i++)
  {
     for(j=0;j<N;j++)
     {
        if(((i*N+j)%salida)==0)
        {
          printf("%.1lf ",a[i*N+j]);
        }
     }
     printf("\n");
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern void sec(int,double *,int,double *,double *,double *);

int main(int argc,char *argv[]) {
//  int nodo,np;
  int i,j,N,P;
  double lr,ur,lm,um;
  int cuantos,semilla,salida;
  long long ti,tf,tt=0;
  double *pol,*meshx,*meshy,*r;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(60);  /* time limit */
 
//  MPI_Init(&argc,&argv);
//  MPI_Comm_size(MPI_COMM_WORLD,&np);
//  MPI_Comm_rank(MPI_COMM_WORLD,&nodo);

// The number of test cases is read
// and sent to all the processes
//  if(nodo==0)
//  {
    scanf("%d",&cuantos);
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
//  else
//  {
//    MPI_Bcast(&cuantos,1,MPI_INT,0,MPI_COMM_WORLD);
//  }
  for(i=0;i<cuantos;i++)
  {
//    if(nodo==0)
//    {
      scanf("%d",&N);                   // The first argument is the size of the matrix for the coefficients
      scanf("%d",&P);                   // Number of points in the mesh PxP
      scanf("%d",&semilla);             // the third is the seed for random generation
      scanf("%lf",&lr);		//lower value to generate the coefficients
      scanf("%lf",&ur);		// upper value to generate the coefficients
      scanf("%lf",&lm);		// lower value for the points in the mesh
      scanf("%lf",&um);		// upper value for the points in the mesh
      scanf("%d",&salida);                 // to write selected values of the output matrix

// Space for the polynomial, the points in the mesh and the result
      pol = (double *) calloc(sizeof(double),N*N);
      meshx = (double *) malloc(sizeof(double)*P*P);
      meshy = (double *) malloc(sizeof(double)*P*P);
      r=(double *) malloc(sizeof(double)*P*P);

      srand(semilla);
      initialize(pol,N,meshx,meshy,P,lr,ur,lm,um);

#ifdef DEBUG
// While debugging the matrix and the valus and positions are written
      escribir(pol,N);
      escribir(meshx,P);
      escribir(meshy,P);
#endif
//    }
//    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds(); 
    sec(N,pol,P,meshx,meshy,r);
//    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
//    if(nodo==0)
//    {
// The time of the first input is not considered
      if(i!=0) tt+=tf-ti;
#ifdef DEBUG
// While debugging the time of each execution is written
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
      escribirresult(r,P,salida);
      free(pol);
      free(meshx);
      free(meshy);
      free(r);
//    } 
  }
  
//  if(nodo==0)
//  { 
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
//  }
//  MPI_Finalize();
  return 0;
}
