#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <mpi.h>
#include <omp.h>
#include <signal.h>
#include <unistd.h>

// Scheme for the problem of Laberint with impulsing positions
// Spanish Parallel Programming Contest 2013, problem F

void initialize(int nw,int nt,int *b,int *c,int seed, int lb,int ub, int pz,int lc,int uc)
{
  int i;
  srand(seed);
// Each possition in the benefit table is randomly generated
  for(i=0;i<nw*nt;i++)
  {  
    b[i]=(int) (((1.*rand())/RAND_MAX)*(ub-lb)+lb);
  }
//some are zeros depending on the percentage
  for(i=0;i<nw*nt;i++)
  {  
    if((int) (((1.*rand())/RAND_MAX)*100)<pz)
      b[i]=0;
  }
// The capacities are randomly generated
  for(i=0;i<nw;i++)
  {  
    c[i]=(int) (((1.*rand())/RAND_MAX)*(uc-lc)+lc);
  }
}

void viewb(int nw,int nt,int *b)
{
  int i, j;

  for (i = 0; i < nw; i++)
  {
    for (j = 0; j < nt; j++)
    {  
      printf("%d ",b[i*nt+j]);
    }
    printf("\n");
  }
  printf("\n");
}

void viewc(int nw,int *c)
{
  int i, j;

  for (i = 0; i < nw; i++)
  {
      printf("%d ",c[i]);
  }
  printf("\n");
}

/*
c
c     mseconds - returns elapsed milliseconds since Jan 1st, 1970.
c
*/
long long mseconds(){
  struct timeval t;
  gettimeofday(&t, NULL);
  return t.tv_sec*1000 + t.tv_usec/1000;
}

static void alarm_handler(int sig) {
  fprintf(stderr, "Time Limit Exceeded\n");
  abort();
}

extern int tasks(int,int,int *,int *,int,int);

int main(int argc,char *argv[]) {
  int node,np; //node indentifier and total number of MPI processes
  int i,j; //auxiliar variables
  int nw,nt; //number of workers and of tasks
  int *b; //checkerboard size and space for the checkerboard
  int *c; //capacities of the workers
  int nproblems; //number of problems to solve
  int seed; //sedd for random generation of data
  int lb,ub,pz,lc,uc; //for the random generation: lower and upper values of the beneficies, percentage of zeros, lower and upper for capacities
  int result;
  long long ti,tf,tt=0;

  FILE *stats_file = fopen("stats", "w");

  struct sigaction sact;
  sigemptyset(&sact.sa_mask);
  sact.sa_flags = 0;
  sact.sa_handler = alarm_handler;
  sigaction(SIGALRM, &sact, NULL);
  alarm(100);  /* time limit */
 
  MPI_Init(&argc,&argv);
  MPI_Comm_size(MPI_COMM_WORLD,&np);
  MPI_Comm_rank(MPI_COMM_WORLD,&node);

// The number of test cases is read
// and sent to all the processes
  if(node==0)
  {
    scanf("%d",&nproblems);
    MPI_Bcast(&nproblems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  else
  {
    MPI_Bcast(&nproblems,1,MPI_INT,0,MPI_COMM_WORLD);
  }
  for(i=0;i<nproblems;i++)
  {
    if(node==0)
   {
      scanf("%d",&nw);		  // The first argument is the number of workers
      scanf("%d",&nt);                   // The second argument is the number of tasks
      scanf("%d",&seed);		  // seed to generate beneficies and capacities
      scanf("%d",&lb);                  //lower value for the benefit
      scanf("%d",&ub);                  //upper value for the benefit
      scanf("%d",&pz);                  //percentage of zeros in the table of benefit
      scanf("%d",&lc);                  //lower value for the capacity
      scanf("%d",&uc);                  //upper value for the capacity
      b=(int *)malloc(sizeof(int)*nw*nt);
      c=(int *) malloc(sizeof(int)*nw);

      initialize(nw,nt,b,c,seed,lb,ub,pz,lc,uc);
#ifdef VIEW
      printf("Initial values:\n Benefits:\n");
      viewb(nw,nt,b);
      printf("Capacities:\n");
      viewc(nw,c);
#endif
    }
    else
    {
       b=NULL;
       c=NULL;
    }
      
    MPI_Barrier(MPI_COMM_WORLD);
    ti=mseconds();
    result=sec(nw,nt,b,c);
    MPI_Barrier(MPI_COMM_WORLD);
    tf=mseconds(); 
    if(node==0)
    {
      
// The time of the first problem is not considered
      if(i!=0) tt+=tf-ti;
#ifdef TIME
// to obtain the execution time of each running
      fprintf(stats_file, "%Ld\n", tf-ti);
#endif
// The results of each problem are written,
// even for the first problem
       printf("%d ",result);
       free(b);
       free(c);
    }
  }
  
  if(node==0)
  { 
    printf("\n");
    fprintf(stats_file, "%Ld\n", tt);
    fclose(stats_file);
  }
  MPI_Finalize();
  return 0;
}
